@@ -23,8 +23,8 @@ from collections import OrderedDict
 
 from .. import mlog
 from .. import mesonlib
-from ..mesonlib import MesonException, Popen_safe, version_compare
-from ..environment import for_windows
+from ..mesonlib import MesonException, Popen_safe, version_compare, extract_as_list
+from ..environment import for_windows, detect_cpu
 
 from .base import DependencyException, DependencyMethods
 from .base import ExternalDependency, ExternalProgram
@@ -291,10 +291,16 @@ class QtBaseDependency(ExternalDependency):
             mincdir = os.path.join(incdir, 'Qt' + module)
             self.compile_args.append('-I' + mincdir)
             if for_windows(self.env.is_cross_build(), self.env):
-                libfile = os.path.join(libdir, self.qtpkgname + module + '.lib')
+                is_debug = self.env.cmd_line_options.buildtype.startswith('debug')
+                dbg = 'd' if is_debug else ''
+                if self.qtver == '4':
+                    base_name = 'Qt' + module + dbg + '4'
+                else:
+                    base_name = 'Qt5' + module + dbg
+                libfile = os.path.join(libdir, base_name + '.lib')
                 if not os.path.isfile(libfile):
                     # MinGW can link directly to .dll
-                    libfile = os.path.join(self.bindir, self.qtpkgname + module + '.dll')
+                    libfile = os.path.join(self.bindir, base_name + '.dll')
                     if not os.path.isfile(libfile):
                         self.is_found = False
                         break
@@ -312,7 +318,7 @@ class QtBaseDependency(ExternalDependency):
             fname = 'Qt' + m
             fwdep = ExtraFrameworkDependency(fname, False, libdir, self.env,
                                              self.language, kwargs)
-            self.cargs.append('-F' + libdir)
+            self.compile_args.append('-F' + libdir)
             if fwdep.found():
                 self.is_found = True
                 self.compile_args += fwdep.get_compile_args()
@@ -384,7 +390,6 @@ class SDL2Dependency(ExternalDependency):
                     return
             except Exception as e:
                 mlog.debug('SDL 2 not found via pkgconfig. Trying next, error was:', str(e))
-                pass
         if DependencyMethods.SDLCONFIG in self.methods:
             sdlconf = shutil.which('sdl2-config')
             if sdlconf:
@@ -426,6 +431,8 @@ class WxDependency(ExternalDependency):
         self.version = 'none'
         if WxDependency.wx_found is None:
             self.check_wxconfig()
+        else:
+            self.wxc = WxDependency.wx_found
         if not WxDependency.wx_found:
             mlog.log("Neither wx-config-3.0 nor wx-config found; can't detect dependency")
             return
@@ -461,12 +468,9 @@ class WxDependency(ExternalDependency):
             self.link_args = out.split()
 
     def get_requested(self, kwargs):
-        modules = 'modules'
-        if modules not in kwargs:
+        if 'modules' not in kwargs:
             return []
-        candidates = kwargs[modules]
-        if not isinstance(candidates, list):
-            candidates = [candidates]
+        candidates = extract_as_list(kwargs, 'modules')
         for c in candidates:
             if not isinstance(c, str):
                 raise DependencyException('wxwidgets module argument is not a string')
@@ -480,9 +484,85 @@ class WxDependency(ExternalDependency):
                     mlog.log('Found wx-config:', mlog.bold(shutil.which(wxc)),
                              '(%s)' % out.strip())
                     self.wxc = wxc
-                    WxDependency.wx_found = True
+                    WxDependency.wx_found = wxc
                     return
             except (FileNotFoundError, PermissionError):
                 pass
         WxDependency.wxconfig_found = False
         mlog.log('Found wx-config:', mlog.red('NO'))
+
+class VulkanDependency(ExternalDependency):
+    def __init__(self, environment, kwargs):
+        super().__init__('vulkan', environment, None, kwargs)
+
+        if DependencyMethods.PKGCONFIG in self.methods:
+            try:
+                pcdep = PkgConfigDependency('vulkan', environment, kwargs)
+                if pcdep.found():
+                    self.type_name = 'pkgconfig'
+                    self.is_found = True
+                    self.compile_args = pcdep.get_compile_args()
+                    self.link_args = pcdep.get_link_args()
+                    self.version = pcdep.get_version()
+                    return
+            except Exception:
+                pass
+
+        if DependencyMethods.SYSTEM in self.methods:
+            try:
+                self.vulkan_sdk = os.environ['VULKAN_SDK']
+                if not os.path.isabs(self.vulkan_sdk):
+                    raise DependencyException('VULKAN_SDK must be an absolute path.')
+            except KeyError:
+                self.vulkan_sdk = None
+
+            if self.vulkan_sdk:
+                # TODO: this config might not work on some platforms, fix bugs as reported
+                # we should at least detect other 64-bit platforms (e.g. armv8)
+                lib_name = 'vulkan'
+                if mesonlib.is_windows():
+                    lib_name = 'vulkan-1'
+                    lib_dir = 'Lib32'
+                    inc_dir = 'Include'
+                    if detect_cpu({}) == 'x86_64':
+                        lib_dir = 'Lib'
+                else:
+                    lib_name = 'vulkan'
+                    lib_dir = 'lib'
+                    inc_dir = 'include'
+
+                # make sure header and lib are valid
+                inc_path = os.path.join(self.vulkan_sdk, inc_dir)
+                header = os.path.join(inc_path, 'vulkan', 'vulkan.h')
+                lib_path = os.path.join(self.vulkan_sdk, lib_dir)
+                find_lib = self.compiler.find_library(lib_name, environment, lib_path)
+
+                if not find_lib:
+                    raise DependencyException('VULKAN_SDK point to invalid directory (no lib)')
+
+                if not os.path.isfile(header):
+                    raise DependencyException('VULKAN_SDK point to invalid directory (no include)')
+
+                self.type_name = 'vulkan_sdk'
+                self.is_found = True
+                self.compile_args.append('-I' + inc_path)
+                self.link_args.append('-L' + lib_path)
+                self.link_args.append('-l' + lib_name)
+
+                # TODO: find a way to retrieve the version from the sdk?
+                # Usually it is a part of the path to it (but does not have to be)
+                self.version = '1'
+                return
+            else:
+                # simply try to guess it, usually works on linux
+                libs = self.compiler.find_library('vulkan', environment, [])
+                if libs is not None and self.compiler.has_header('vulkan/vulkan.h', '', environment):
+                    self.type_name = 'system'
+                    self.is_found = True
+                    self.version = 1 # TODO
+                    for lib in libs:
+                        self.link_args.append(lib)
+                    return
+
+    def get_methods(self):
+        return [DependencyMethods.PKGCONFIG, DependencyMethods.SYSTEM]
