@@ -90,7 +90,7 @@ def get_backend_commands(backend, debug=False):
         test_cmd = cmd + ['-target', 'RUN_TESTS']
     elif backend is Backend.ninja:
         # We need at least 1.6 because of -w dupbuild=err
-        cmd = [detect_ninja('1.6'), '-w', 'dupbuild=err']
+        cmd = [detect_ninja('1.6'), '-w', 'dupbuild=err', '-d', 'explain']
         if cmd[0] is None:
             raise RuntimeError('Could not find Ninja v1.6 or newer')
         if debug:
@@ -104,6 +104,12 @@ def get_backend_commands(backend, debug=False):
     return cmd, clean_cmd, test_cmd, install_cmd, uninstall_cmd
 
 def ensure_backend_detects_changes(backend):
+    # We're using a ninja with QuLogic's patch for sub-1s resolution timestamps
+    # and not running on HFS+ which only stores dates in seconds:
+    # https://developer.apple.com/legacy/library/technotes/tn/tn1150.html#HFSPlusDates
+    # FIXME: Upgrade Travis image to Apple FS when that becomes available
+    if 'MESON_FIXED_NINJA' in os.environ and not mesonlib.is_osx():
+        return
     # This is needed to increase the difference between build.ninja's
     # timestamp and the timestamp of whatever you changed due to a Ninja
     # bug: https://github.com/ninja-build/ninja/issues/371
@@ -171,7 +177,7 @@ if __name__ == '__main__':
                 backend = Backend.xcode
             break
     # Running on a developer machine? Be nice!
-    if not mesonlib.is_windows() and 'TRAVIS' not in os.environ:
+    if not mesonlib.is_windows() and not mesonlib.is_haiku() and 'TRAVIS' not in os.environ:
         os.nice(20)
     # Appveyor sets the `platform` environment variable which completely messes
     # up building with the vs2010 and vs2015 backends.
